#!/usr/local/bin/python
from __future__ import print_function, division
import sys
import numpy as np

import matplotlib as mpl
import matplotlib.pyplot as plt
import matplotlib.cm as cm
from matplotlib.ticker import AutoMinorLocator

import scipy.stats as stats

class Point(object):
    def __init__(self, parentID, x, y, theta, ke):
        self.parentID = parentID
        self.x = x
        self.y = y
        self.theta = theta
        self.ke = ke


n = int(sys.argv[1])

sys.stdout.write("Plot z = " + str(n) + " mm... \n")
sys.stdout.flush()

cb_ticks = 5

### import data
filename = 'data/output_z' + str(n) + '.txt'

f = open(filename)

x = []
y = []
theta = []
ke = []


dp_list = []

npoints = 0

for line in f.readlines():
    if line[0] != "#":
      
        npoints += 1 
        line = line.split()
        
        parentID_i = int(line[0])
        
        x_i     = float(line[1])
        y_i     = float(line[2])
        theta_i = float(line[4])
        ke_i    = float(line[5])
        
        if parentID_i == 0:
            x.append(x_i)
            y.append(y_i)
            theta.append(theta_i)
            ke.append(ke_i)


x     = np.array(x)
y     = np.array(y)
theta = np.array(theta)
ke    = np.array(ke)



fig = plt.figure(1)

##----------------##
##   PARAMETERS   ##
##----------------##

zoom = 4 if n <= 800 else 2


##-------------##
##   PROFILE   ##
##-------------##

size = 1/zoom

lim = 100

nbins = int(1000/size)
bins = np.linspace(-lim, lim, nbins)

density, xedges, yedges = np.histogram2d(x, y, bins)
extent = [-lim, lim, -lim, lim]

plt.subplot(221)
ax = plt.gca()

plt.title("Profile, z = %d mm" % (n))

plt.xlim(-lim*size, lim*size)
plt.ylim(-lim*size, lim*size)

plt.xlabel(r'$x$ [mm]')
plt.ylabel(r'$y$ [mm]')

plt.grid(color='w')

plt.tick_params(which='major', color='w')
plt.tick_params(which='minor', color='w')

ax.xaxis.set_minor_locator(AutoMinorLocator(2))
ax.yaxis.set_minor_locator(AutoMinorLocator(2))


# colormap usage: cmap=cm.get_cmap('ocean_r')
#     - good maps:  ocean_r, cubehelix, inferno, magma, CMRmap
#     - favourites: (1) CMRmap, (2) inferno
#
#     --> use ocean_r to see extent of halo and then CMRmap to see gradient in bulk


plt.imshow(density.T, cmap=cm.get_cmap('CMRmap'), extent=extent, interpolation=None, origin='lower')

ticks = np.linspace(0, 0.95*np.max(density), cb_ticks)
ticks = np.around(ticks, decimals=0)
cb = plt.colorbar(ticks=ticks)

cb.formatter.set_powerlimits((0, 0))
cb.ax.yaxis.set_offset_position('left')
cb.update_ticks()


##---------------##
##   EMITTANCE   ##
##---------------##

thetazoom = 8 if n < 50 else 2

size = 1/zoom

xlim = 100
thetalim = 200

nbins = int(2000)
bins  = np.linspace(-xlim, xlim, nbins)
ybins = np.linspace(-thetalim, thetalim, nbins)

density, xedges, yedges = np.histogram2d(x, theta, [bins, ybins])
extent = [-xlim, xlim, -thetalim, thetalim]

plt.subplot(223)

plt.title("Emittance")
ax = plt.gca()

plt.xlim(-xlim*size, xlim*size)
plt.ylim(-thetalim*size/thetazoom, thetalim*size/thetazoom)

plt.xlabel(r'$x$ [mm]')
plt.ylabel(r"$x'$ [mrad]")

plt.grid(color='w')

plt.tick_params(which='major', color='w')
plt.tick_params(which='minor', color='w')

ax.xaxis.set_minor_locator(AutoMinorLocator(2))
ax.yaxis.set_minor_locator(AutoMinorLocator(2))

aspect = xlim/(thetalim/thetazoom)

plt.imshow(density.T, cmap=cm.get_cmap('CMRmap'), extent=extent, origin='lower', aspect=aspect)

ticks = np.linspace(0, 0.95*np.max(density), cb_ticks)
ticks = np.around(ticks, decimals=0)
cb = plt.colorbar(ticks=ticks)

cb.formatter.set_powerlimits((0, 0))
cb.ax.yaxis.set_offset_position('left')
cb.update_ticks()

#plt.show()



##----------------------------##
##   PROJECTION ONTO X-AXIS   ##
##----------------------------##

plt.subplot(222)
ax = plt.gca()

plt.title("Projection onto x-axis")

plt.xlabel(r'$x$ [mm]')
plt.ylabel(r'n$^{\rm o}$ of protons')

xprojection = np.array([np.sum(density[i,:]) for i in range(nbins-1)])
plt.scatter(bins[:-1], xprojection, s=5, facecolor='b', lw = 0)

# grid
plt.grid()
ax.xaxis.grid(which='minor')

# ticks
ax.xaxis.set_minor_locator(AutoMinorLocator(2))
ax.yaxis.set_minor_locator(AutoMinorLocator(2))

plt.ticklabel_format(style='sci', axis='y', scilimits=(0,0))

# plot limits
xlim = 100

size = 1/zoom
plt.xlim(-xlim*size, xlim*size)
plt.ylim(0, plt.ylim()[1])


# box

textstr = "{0:8s}{1:>.3e}".format(r'$N_{total} =$', np.sum(xprojection))

#xbox = 0.46 if mean > 61.5 else 0.95
xbox = 0.44
ybox = 0.95

ax.text(xbox, ybox, textstr, transform=ax.transAxes, bbox={'facecolor':'w', 'pad':2}, va='top', ha='right')



##---------------------##
##   ENERGY SPECTRUM   ##
##---------------------##

nbins = 500

fit_min = 57.5
fit_max = 65

ke = np.sort(ke)
ke_tofit = np.array([ke_i for ke_i in ke if (fit_min <= ke_i <= fit_max)])


### create subplot ###

plt.subplot(224)
ax = plt.gca()

plt.title(" Kinetic energy spectrum")

### plot histogram ###

ke_hist = plt.hist(ke, bins=nbins, histtype='stepfilled', alpha=0.5, color='b')
ke_hist = ke_hist[0]

ke_pdf = np.linspace(fit_min, fit_max, nbins)

# get mean and std deviation of the fit
mean, stddev = stats.norm.fit(ke_tofit)
fit = stats.norm.pdf(ke_pdf, mean, stddev)

# normalise the fit, then scale it
fit /= np.max(fit)
scaled_fit = fit * np.max(ke_hist)


plt.xlabel(r'Energy [MeV]')
plt.ylabel(r'n$^{\rm o}$ of protons')

nentries = int(len(ke))

textstr = "{0:<10s}{1:>6.2f}\n{2:<10s}{3:>6.3f}".format("Mean", mean, "Std dev", stddev)

xbox = 0.46 if mean > 61.5 else 0.95
ybox = 0.95

ax.text(xbox, ybox, textstr, transform=ax.transAxes, bbox={'facecolor':'w', 'pad':2}, va='top', ha='right')


### plot Gaussian fit ###

plt.plot(ke_pdf, scaled_fit, 'r', lw=1)

# ticks
ax.xaxis.set_minor_locator(AutoMinorLocator(2))
plt.ticklabel_format(style='sci', axis='y', scilimits=(0,0))


plt.xlim(fit_min-2.5, fit_max+2.5)
plt.ylim(0, plt.ylim()[1]*1.1)

plt.tight_layout(pad=0.5)
fig.subplots_adjust(left=-1e-4) # or -1e-4 or 0.02

# grid
#plt.grid()
#ax.xaxis.grid(which='minor')


##-----------------##
##   SAVE FIGURE   ##
##-----------------##

#plt.show()

savefile = 'plots/tiles_' + str(n) + '.png'

#savefile = 'plots/emittance/plot_emittance_z_' + str(n) + '.png'

#savefile = 'plots/profile/plot_emittance_z_' + str(n) + '.png'
plt.savefig(savefile, dpi=300)
#plt.close()

sys.stdout.write("done\n")
