# This script allows simulations to be run in parallel. It submits a given
# number of simulations to a given queue. If input is read from a file, use
# the -r flag to indicate this is the case. Input files should be split
# into the required number of chunks using the split_input.sh script beforehand.
# Usage:
#   ./submit.sh queue nsimulations [-r, --read]
# ------------------------------------------------------------------------------
# Author: 07/07/2017, M. Hentz
# Modified:
#   07/03/2018, M. Hentz
# ------------------------------------------------------------------------------
#!/bin/bash

# Check if SL6 env is set.
# Warning: This may have to be updated when upgrading to CentOS 7.
env=$(printenv | grep "PBTENV")
if [[ "$env" != "PBTENV=SL6 Production" ]]; then
    echo "Not in SL6 Production environment."
	  echo "Set it by running: $ /unix/pbt/software/scripts/pbt.sh"
    #exit
fi

usage="Usage: ./submit.sh queue nsimulations [-r, --read]"

# Check if arguments were given
if [[ -z $1 ]]; then
    echo -n "No arguments supplied. Need to specify the"
    echo    " queue and number of simulations required."
	  echo    $usage
    exit
fi

# Check if flag is set to read input from a file
read_input=0
while [[ $# -gt 0 ]]; do
    key="$1"
    case $key in
        -r|--read)
        read_input=1
        shift # past argument
        shift
        ;;
        *)    # unknown option
        positional+=("$1") # save it in an array for later
        shift # past argument
        ;;
    esac
done
set -- "${positional[@]}" # restore positional parameters

# Get the queue and number of simulations
queue=${positional[0]}
nsimulations=${positional[1]}

# read_input is true
if [[ $read_input -eq 1 ]]; then
	  echo " Submitting $nsimulations simulations..."
fi

# read_input is false
if [[ $read_input -eq 0 ]]; then
	# Get the number of events specified in the macro for parallel simulations
	nevents=$(grep '/run/beamOn' ../proton.mac | sed 's/[^0-9]*//g')
	echo " Submitting $nsimulations simulations of $nevents events..."
fi


# Set simulation directory name
sim_dir=$(pwd | cut -d "_" -f 2 | cut -d "/" -f 1)

# Submit $nsimulations jobs
for ((i=1; i<=$nsimulations; i++)); do
    # If input is read from a file
	  if [[ $read_input -eq 1 ]]; then
	      # Generate submission script from template cb_parallel.sh
		    # Seed values are calculated in the submission script using i
	      cat ../cb_parallel.sh | sed -e "s/\${i}/$i/" > cb_parallel_$i.tmp

        # Count number of lines in input which is the number of primaries
		    NLINES=$(echo $(wc -l ../read/$i.in | awk '{ printf $1; }'))

		    # Substitute number of input particles into temporary cb_parallel.sh
		    cat cb_parallel_$i.tmp | sed -e "s/\${nlines}/${NLINES}/" \
            > cb_parallel_$i.sh
	      rm cb_parallel_$i.tmp
	  fi

    # If primaries are generated using macro settings only assign seeds
	  if [[ $read_input -eq 0 ]]; then
	      cat ../cb_parallel.sh | sed -e "s/\${i}/$i/" > cb_parallel_$i.sh
	  fi

	  # Submit job
	  jobname=$(printf "%03d_cb${sim_dir}" $i)
    qsub -q $queue -N $jobname cb_parallel_$i.sh

    # Delete submission script
	  rm cb_parallel_$i.sh
done
